!******************************************************************************!
!                                                                              !
!    reset_rcl.f90                                                             !
!    is part of RECOLA2 (REcursive Computation of One Loop Amplitudes)2        !
!                                                                              !
!    Copyright (C) 2016,2017 Ansgar Denner, Jean-Nicolas Lang and              !
!                            Sandro Uccirati                                   !
!                                                                              !
!    RECOLA2 is licenced under the GNU GPL version 3,                          !
!    see COPYING for details.                                                  !
!                                                                              !
!******************************************************************************!

module reset_rcl

!------------------------------------------------------------------------------!

  use globals_rcl
  use amplitude_rcl
  use tables_rcl
  use order_rcl

!------------------------------------------------------------------------------!

  implicit none

!------------------------------------------------------------------------------!

  contains

!------------------------------------------------------------------------------!

  subroutine deallocate_rcl

  integer :: i,n
  logical :: fileopen
  ! This subroutine deallocates all global allocatable arrays.
  ! It has to be called at the end of the program.

  deallocate (timeTI,timeTC,inpr,legsIn,legsOut,par,hel,resMax,      &
              binRes,parRes,powsel,loop,qflow,process)
  if (allocated(momenta))      deallocate(momenta)
  if (allocated(matrix))       deallocate(matrix)
  if (allocated(matrix2))      deallocate(matrix2)
  if (allocated(matrix2h))     deallocate(matrix2h)
  if (allocated(matrix2int))   deallocate(matrix2int)
  if (allocated(matrix2cc))    deallocate(matrix2cc)
  if (allocated(matrix2ccint)) deallocate(matrix2ccint)
  if (allocated(matrix2scc))   deallocate(matrix2scc)
  if (allocated(matrix2sc))    deallocate(matrix2sc)
  deallocate (factor,newleg,oldleg,newbin,oldbin,defp2bin,p2bin,    &
              defresbin,pspbin,mONS,cmONS2,cmREG2,cfTot,csTot,      &
              pCsTot,csAmp,csIq,nAmp,pAmp,facAmp, w0eTot,he,        &
              dualconf,lpmax,cd0sMax,w0Tot,w0last,parw0e,binw0e,    &
              legw0e,helw0e,c0EffMax,modaTot, zeroLO,               &
              colcoef,colcoefc,mosm0,binsm0,parsm0,xsm0,            &
              gsIncsm0,cosm0,gssm0,cssm0,dasd0,sesd0,facsd0,gssd0,  &
              cssd0,c0TOlp,bm0min,bm0max,bd0min,bd0max,sbm0,w0inbm0,&
              w0outbm0,winitbm0,typebm0,sbd0,w0outbd0,winitbd0)

  if (loopMax) deallocate (loopCoef,cEffMax,tiTot,ritiMax,          &
                           mosm1,binsm1,parsm1,gsIncsm1,cosm1,      &
                           rankInsm1,rankOutsm1,ferloopsm1,gssm1,   &
                           cssm1,tism1,dasd1,facsd1,rankOutsd1,     &
                           ferloopsd1,gssd1,cssd1,tisd1,legsti,     &
                           momsti,vmti,rankti,cTOt,cTOfh,cTOih1,    &
                           bm1min,bm1max,sbm1,w1inbm1,w0inbm1,      &
                           w1outbm1,winitbm1,typebm1,bd1min,bd1max, &
                           sbd1,w1outbd1,winitbd1,w1TotMax,riwMax)
  if (allocated(nCache)) deallocate(nCache)
  if (allocated(nCacheTot)) deallocate(nCacheTot)
  if (allocated(tiCache)) deallocate(tiCache)
  if (allocated(cacheOn)) deallocate(cacheOn)

  ! tables
  deallocate (levelLeg,vectorLeg,firstNumber,firstGap,firstNumbers, &
              firstGaps)
  if (loopMax) deallocate (RtoS,riMin,riMax,ri,RItoR,RItoI,incRI,   &
                           firstRI)
  if (allocated(als0R)) deallocate(als0R,dZgs0R,Qren0R,Nlq0R)

  ! order_keys
  if (allocated(oi_reg)) deallocate(oi_reg)
  if (allocated(oi2_reg)) deallocate(oi2_reg)
  if (allocated(max_od)) deallocate(max_od)
  if (allocated(compoi)) deallocate(compoi)
  oiDef = 10
  if (allocated(oiSize)) deallocate(oiSize)
  if (allocated(oi2Size)) deallocate(oi2Size)

! Set internal variables to default values
  resIR = .false.
  prTot = 0
  loopMax = .false.
  processes_generated = .false.
  ifail = 0

  do i = 1,nOpened
    inquire(file=trim(nameOpened(i)),opened=fileopen,number=n)
    if (fileopen) close(n)
  enddo
  outputfile = 'output.rcl'
  nx = 934758

  call deallocate_vertices_rcl()
  call deallocate_particles_rcl()
  
  end subroutine deallocate_rcl

!------------------------------------------------------------------------------!

  subroutine deallocate_vertices_rcl()
    use modelfile, only: clear_vertices_mdl
    call clear_vertices_mdl
  end subroutine deallocate_vertices_rcl

!------------------------------------------------------------------------------!

  subroutine deallocate_particles_rcl()
    use modelfile, only: clear_particles_mdl
    call clear_particles_mdl
  end subroutine deallocate_particles_rcl

!------------------------------------------------------------------------------!

  subroutine reset_recola_rcl

    call deallocate_rcl

  end subroutine reset_recola_rcl

!------------------------------------------------------------------------------!

end module reset_rcl

!------------------------------------------------------------------------------!
